/*
 * PSP Timer
 *
 * Timer for the Personal Software Process
 *
 * Copyright 1997 David Byers
 *
 */

#include <malloc.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/Separator.h>

#include "report.h"
#include "askuser.h"
#include "timer.h"


char            gGlobalPause;
tTimer*         gTimerList[NUM_TIMERS];
XtIntervalId    gActiveTimeout;
XtAppContext    gAppContext;
Pixmap          gClockPixmapArray[16];
Pixmap          gCheckedPixmap, gUncheckedPixmap;
Widget          gTotalTimeWidget, gPauseButton, gLeftFooter, gRightFooter;
ulong           gTotalTime;
GC              gIndicatorGC;
Widget          gMainWindow;

String          gFallbackResources[] = {
    "PSPTimer*background:               gray",
    "PSPTimer*timerWindow*fontList:     -*-lucida-*-r-*-sans-10-*-*-*-*-*-*-*",
    "PSPTimer*footerForm*fontList:     -*-lucida-*-r-*-sans-10-*-*-*-*-*-*-*",
    "PSPTimer*fontList:                 -*-lucida-*-r-*-sans-12-*-*-*-*-*-*-*"
    "PSPTimer*timerWindow*marginHeight: 0",
    "PSPTimer*timerWindow*marginWidth:  0",
    "PSPTimer.title:                    PSP Timer",
    "PSPTimer.height:                   380",
    "PSPTimer.width:                    395",
    "PSPTimer*eventLogWindow.title:     Timer Log",
    "PSPTimer*eventLogTable*fontList:    -*-courier-medium-r-*-*-10-*-*-*-*-*-*-*",
    "PSPTimer*eventLogTable.spacing:    0",
    "PSPTimer*eventLogTable*XmTextField*marginHeight: 1",
    "PSPTimer*eventLogTable*XmTextField*shadowThickness: 1",
    "PSPTimer*eventLogTableHeader*fontList:    -*-courier-bold-r-*-*-10-*-*-*-*-*-*-*",
    "PSPTimer*eventLogWindow.width:     535",
    "PSPTimer*eventLogWindow.height:    144",
    NULL
};


void MaybeMakeGC(Widget widget)
{
    XGCValues       gcData;
    
    if (gIndicatorGC == 0)
    {
        XtVaGetValues(widget,
                      XmNforeground, &gcData.foreground,
                      XmNbackground, &gcData.background,
                      NULL);
        gIndicatorGC = XCreateGC(XtDisplay(widget),
                                 XtWindow(widget),
                                 GCForeground | GCBackground,
                                 &gcData);
    }
}


void ReadPixmaps(Screen *s, Pixel fg, Pixel bg)
{
    char *fileName;
    int   i;

    fileName = malloc(1 +
                      strlen(PIXMAP_DIRECTORY) +
                      strlen("uchk.xbm"));
    if (fileName == NULL)
    {
        perror("malloc");
        exit(1);
    }
                      
    
    for (i = 0; i < 16; i++)
    {
        sprintf(fileName,
                "%st%02d.xbm",
                PIXMAP_DIRECTORY,
                i);
        gClockPixmapArray[i] = XmGetPixmap(s, fileName, fg, bg);
    }

    sprintf(fileName,
            "%s%s",
            PIXMAP_DIRECTORY,
            "chk.xbm");
    gCheckedPixmap = XmGetPixmap(s, fileName, fg, bg);
    sprintf(fileName,
            "%s%s",
            PIXMAP_DIRECTORY,
            "uchk.xbm");
    gUncheckedPixmap = XmGetPixmap(s, fileName, fg, bg);

    free(fileName);
}

void SetLabel(Widget widget, char *label)
{
    XmString    tmpString;

    tmpString = XmStringCreateSimple(label);
    XtVaSetValues(widget,
                  XmNlabelString, tmpString,
                  NULL);
    XmStringFree(tmpString);
}

void SetLabelPixmap(Widget widget, Pixmap label)
{
    XtVaSetValues(widget,
                  XmNlabelPixmap, label,
                  NULL);
}


void SetLeftFooter(char *s)
{
    SetLabel(gLeftFooter, s);
}


void SetRightFooter(char *s)
{
    SetLabel(gRightFooter, s);
}


void RedrawIndicator(Widget widget,
                     XtPointer clientData,
                     XtPointer callbackData)
{
    tTimer  *timer;
    Window   rootWindow;
    int      x, y;
    uint     width, height, depth, borderWidth;

    XGetGeometry(XtDisplay(widget),
                 XtWindow(widget),
                 &rootWindow,
                 &x, &y,
                 &width, &height, &borderWidth, &depth);
                 
    y = (height - 16) >> 1;

    MaybeMakeGC(widget);
    timer = (tTimer *)clientData;
    XCopyArea(XtDisplay(widget),
              gClockPixmapArray[timer->pixmapIndex],
              XtWindow(widget),
              gIndicatorGC,
              0, 0, 16, 16, 0, y);
}


void UpdateTotalTime(void)
{
    ulong       total;
    int         i;
    char    buf[9];

    total = 0;
    for (i = 0; i < NUM_TIMERS; i++)
    {
        if (gTimerList[i]->exclusive)
            total += gTimerList[i]->timerValue;
    }

    if (total != gTotalTime)
    {
        gTotalTime = total;
        sprintf(buf,
                "%04ld:%02ld",
                (long int)floor((double)gTotalTime / 60.0),
                (gTotalTime % 60));
        
        SetLabel(gTotalTimeWidget, buf);


    }
}

void UpdateTimerLabels(tTimer *timer)
{
    char    buf[9];

    if (timer->needUpdate)
    {
        sprintf(buf,
                "%04ld:%02ld",
                (long int)floor(timer->timerValue / 60.0),
                timer->timerValue % 60);

        SetLabel(timer->timeDisplayWidget, buf);
    }
    
    RedrawIndicator(timer->indicatorWidget,
                    (XtPointer) timer,
                    NULL);
    timer->needUpdate = 0;
}

void UpdatePercentages()
{
    char buf[5];
    int i, percent;

    for (i = 0; i < NUM_TIMERS; i++)
    {
        if (gTimerList[i]->exclusive)
        {
            percent = (gTotalTime == 0) ? 0 :
                (int)rint((((double)gTimerList[i]->timerValue) /
                      ((double)gTotalTime)) * 100.0);
            if (percent != gTimerList[i]->percentValue)
            {
                gTimerList[i]->percentValue = percent;
                sprintf(buf, "% 3d%%", percent);
                SetLabel(gTimerList[i]->percentDisplayWidget, buf);
            }
        }
    }
}



void TimerCallback(XtPointer clientData, XtIntervalId *id)
{
    int     i;
    int     haveActive = 0, didTick = 0;

    gActiveTimeout = 0;
    if (gGlobalPause == 0)
    {
        for (i = 0; i < NUM_TIMERS; i++)
        {
            if (gTimerList[i]->enabled)
            {
                gTimerList[i]->pixmapIndex =
                    (gTimerList[i]->pixmapIndex + 1) % 16;

                if (gTimerList[i]->pixmapIndex % 8 == 0)
                {
                    gTimerList[i]->timerValue += 1;
                    gTimerList[i]->needUpdate = 1;
                    didTick = 1;
                }
                haveActive = 1;
            }
        }
    
        if (haveActive)
        {
            UpdateTotalTime();
            for (i = 0; i < NUM_TIMERS; i++)
                if (gTimerList[i]->enabled)
                    UpdateTimerLabels(gTimerList[i]);
            if (didTick)
                UpdatePercentages();
            
            gActiveTimeout = XtAppAddTimeOut(gAppContext,
                                             125,
                                             TimerCallback,
                                             clientData);
        }
    }
}


void AddTimeout(void)
{
    if (gActiveTimeout == 0)
        gActiveTimeout = XtAppAddTimeOut(gAppContext,
                                         125,
                                         TimerCallback,
                                         NULL);
}

void RemoveTimeout(void)
{
    if (gActiveTimeout != 0)
    {
        XtRemoveTimeOut(gActiveTimeout);
        gActiveTimeout = 0;
    }
}

void ResetTimer(tTimer *timer)
{
    timer->timerValue = 0;
    timer->pixmapIndex = 0;
    if (timer->enabled == 0)
        SetLabel(timer->controlWidget, "Start");
    timer->needUpdate = 1;
    UpdateTimerLabels(timer);
    UpdateTotalTime();
    UpdatePercentages();
}

void DisableTimer(tTimer *timer)
{
    timer->enabled = 0;
    SetLabel(timer->controlWidget, "Start");
}

void DisableAllTimers(void)
{
    int i;

    for (i = 0; i < NUM_TIMERS; i++)
        DisableTimer(gTimerList[i]);
}

void EnableTimer(tTimer *timer)
{
    ulong   i;

    timer->enabled = 1;
    if (timer->exclusive)
    {
        for (i = 0; i < NUM_TIMERS; i++)
        {
            if (gTimerList[i]->enabled &&
                gTimerList[i]->exclusive &&
                gTimerList[i] != timer)
            {
                DisableTimer(gTimerList[i]);
            }
            if (gTimerList[i]->exclusive &&
                gTimerList[i]->pixmapIndex &&
                gTimerList[i] != timer)
            {
                gTimerList[i]->pixmapIndex = 0;
                UpdateTimerLabels(gTimerList[i]);
            }
        }
    }

    if (timer->eventType != kNone)
        NewEvent(timer->eventType);

    SetLabel(timer->controlWidget, "Stop");
}

void ToggleGlobalPause(void)
{
    if (gGlobalPause)
    {
        EndInterruption();
        gGlobalPause = 0;
        SetLabel(gPauseButton, "Pause all");
    }
    else
    {
        StartInterruption();
        gGlobalPause = 1;
        SetLabel(gPauseButton, "Resume all");
    }
}


void StopEverything(void)
{
    int     i;

    RemoveTimeout();
    for (i = 0; i < NUM_TIMERS; i++)
    {
        gTimerList[i]->pixmapIndex = 0;
        gTimerList[i]->enabled = 0;
        ResetTimer(gTimerList[i]);
    }
    if (gGlobalPause)
        ToggleGlobalPause();
    AddTimeout();
}

void GlobalPauseCallback(Widget w,
                         XtPointer clientData,
                         XtPointer callbackData)
{
    RemoveTimeout();
    ToggleGlobalPause();
    AddTimeout();
}


void TimerControlCallback(Widget w,
                          XtPointer clientData,
                          XtPointer callbackData)
{
    tTimer  *timer;

    RemoveTimeout();
    timer = (tTimer *)clientData;
    if (timer->enabled)
    {
        DisableTimer(timer);
        if (timer->exclusive)
            CloseEvent();
    }
    else
    {
        EnableTimer(timer);
    }
    AddTimeout();
}


void TimerResetCallback(Widget w,
                        XtPointer clientData,
                        XtPointer callbackData)
{
    tTimer  *timer;

    RemoveTimeout();
    timer = (tTimer *)clientData;
    ResetTimer(timer);
    AddTimeout();
}


char ConfirmIfDirty(void)
{
    if (gEventLogIsDirty)
    {
        return AskUser("The current event log has been changed\n"
                       "since last time it was saved. The command\n"
                       "you selected will erase the current event\n"
                       "log without saving it first.\n\n"
                       "Do you want to continue anyway?",
                       "Continue",
                       "Don't continue",
                       NO);
    }
    else
        return 1;
}



void FileMenuCallback(Widget widget,
                      int item)
{
    extern void Testreport(void);

    switch (item)
    {
    case 0:                     /* New */
    {
        char ans = 1;
    
        if (!ConfirmIfDirty())
            break;

        if (ans)
        {
            StopEverything();
            ResetEventLog();
        }
        break;
    }
    case 1:                     /* Open */
    {
        char *fileName;
        int i;

        if (!ConfirmIfDirty())
            break;

        fileName = SelectFile("Open", 1);
        if (fileName == NULL)
            break;

        StopEverything();

        if (ReadEventFile(fileName))
        {
            SetEventLogFilename(fileName);
            for (i = 0; i < NUM_TIMERS; i++)
            {
                gTimerList[i]->needUpdate = 1;
                UpdateTimerLabels(gTimerList[i]);
            }
            UpdateTotalTime();
            UpdatePercentages();
        }
        
        free(fileName);

        break;
    }
    case 2:                     /* Save */
    case 3:                     /* Save as */
    {
        char *fileName, *oldFileName;

        DisableAllTimers();
        oldFileName = GetEventLogFilename();
        if (oldFileName != NULL)
            oldFileName = strdup(oldFileName);
        
        if (item == 3 || oldFileName == NULL)
        {
            fileName = SelectFile("Save as", 0);
            if (fileName == NULL)
            {
                if (oldFileName)
                    free(oldFileName);
                break;
            }
            SetEventLogFilename(fileName);
        }

        if (!WriteEventFile(GetEventLogFilename()))
            SetEventLogFilename(oldFileName);
        else
            gEventLogIsDirty = 0;
        if (oldFileName)
            free(oldFileName);

        break;
    }
    case 4:                     /* Edit report */
        ViewEventLogWindow();
        break;
    case 5:                     /* Save report */
    {
        char *fileName;

        DisableAllTimers();
        fileName = SelectFile("Save report as", 0);
        if (fileName == NULL)
            break;
        SaveReport(fileName);
        break;
    }
    case 6:                     /* Save LaTeX */
    {
        char *fileName;

        DisableAllTimers();
        fileName = SelectFile("Save LaTeX as", 0);
        if (fileName == NULL)
            break;
        SaveTeXReport(fileName);
        break;

    }
    case 7:                     /* Print report */
    {
        char        *cmd;
        
        cmd = AskUserForPrinter();
        if (cmd)
        {
            PrintReport(cmd);
            XtFree(cmd);
        }
        break;
    }
    case 8:                     /* Quit */
    {
        char ans = 1;

        if (gEventLogIsDirty)
        {
            ans = AskUser("The current event log has been modified since\n"
                          "last time it was saved.\n\n"
                          "Are you sure you want to quit?",
                          "Quit",
                          "Don't quit",
                          NO);
        }
        if (ans)
            exit(0);
        break;
    }
    }
}


tTimer  *CreateTimer(Widget parent,
                     tTimer *prev,
                     char *timerName,
                     char *timerLabel,
                     char exclusive,
                     tEventType eventType)
{
    tTimer      *tmp;

    tmp = malloc(sizeof(tTimer));
    if (tmp == NULL)
    {
        perror("malloc");
        exit(1);
    }

    tmp->timerValue = 0;
    tmp->percentValue = 0;
    tmp->pixmapIndex = 0;
    tmp->enabled = 0;
    tmp->exclusive = exclusive;
    tmp->eventType = eventType;

    tmp->idiotForm =
        XtVaCreateManagedWidget("compoundForm",
                                xmFormWidgetClass,
                                parent,
                                XmNmarginHeight, 0,
                                XmNmarginWidth, 0,
                                NULL);


    tmp->indicatorWidget =
        XtVaCreateManagedWidget("indicator",
                                xmDrawingAreaWidgetClass,
                                tmp->idiotForm,
                                XmNleftAttachment, XmATTACH_FORM,
                                XmNtopAttachment, XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNheight, 16,
                                XmNwidth, 16,
                                NULL);

    XtAddCallback(tmp->indicatorWidget,
                  XmNexposeCallback,
                  RedrawIndicator,
                  tmp);

    tmp->nameWidget =
        XtVaCreateManagedWidget("timerLabel",
                                xmLabelWidgetClass,
                                tmp->idiotForm,
                                XmNalignment, XmALIGNMENT_END,
                                XmNrightAttachment, XmATTACH_FORM,
                                XmNtopAttachment, XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNleftAttachment, XmATTACH_WIDGET,
                                XmNleftWidget, tmp->indicatorWidget,
                                XmNwidth, 100,
                                NULL);

    tmp->displayForm =
        XtVaCreateManagedWidget("timerDisplay",
                                xmFormWidgetClass,
                                parent,
                                XmNmarginHeight, 0,
                                XmNmarginWidth, 0,
                                NULL);

    tmp->timeDisplayWidget =
        XtVaCreateManagedWidget("timerTime",
                                xmLabelWidgetClass,
                                tmp->displayForm,
                                XmNalignment, XmALIGNMENT_BEGINNING,
                                XmNleftAttachment, XmATTACH_FORM,
                                XmNtopAttachment, XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                NULL);

    tmp->percentDisplayWidget =
        XtVaCreateManagedWidget("timerPercent",
                                xmLabelWidgetClass,
                                tmp->displayForm,
                                XmNalignment, XmALIGNMENT_END,
                                XmNrightAttachment, XmATTACH_FORM,
                                XmNtopAttachment, XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNrightOffset, 5,
                                NULL);

    tmp->controlForm =
        XtVaCreateManagedWidget("controlForm",
                                xmRowColumnWidgetClass,
                                parent,
                                XmNpacking, XmPACK_TIGHT,
                                XmNorientation, XmHORIZONTAL,
                                XmNmarginHeight, 0,
                                XmNmarginWidth, 0,
                                NULL);

    tmp->controlWidget =
        XtVaCreateManagedWidget("pauseButton",
                                xmPushButtonWidgetClass,
                                tmp->controlForm,
                                XmNalignment, XmALIGNMENT_CENTER,
                                XmNwidth, 60,
                                XmNmarginWidth, 10,
                                XmNrecomputeSize, False,
                                NULL);

    if (exclusive == 0)
    {
        tmp->resetWidget =
            XtVaCreateManagedWidget("resetButton",
                                    xmPushButtonWidgetClass,
                                    tmp->controlForm,
                                    XmNalignment, XmALIGNMENT_CENTER,
                                    XmNmarginWidth, 10,
                                    NULL);
        SetLabel(tmp->resetWidget, "Reset");
        XtAddCallback(tmp->resetWidget,
                      XmNactivateCallback,
                      TimerResetCallback,
                      (XtPointer)tmp);
    }




    SetLabel(tmp->nameWidget, timerLabel);
    SetLabel(tmp->timeDisplayWidget, "0000:00");
    if (exclusive)
        SetLabel(tmp->percentDisplayWidget, "  0%");
    else
        SetLabel(tmp->percentDisplayWidget, "");

    SetLabel(tmp->controlWidget, "Start");
    XtAddCallback(tmp->controlWidget,
                  XmNactivateCallback,
                  TimerControlCallback,
                  (XtPointer)tmp);

    return tmp;
}
                          

void BuildMenus(Widget mainWindow)
{
    Widget      menubar;
    XmString    fileString;
    XmString    openString, saveString, saveAsString, quitString;
    XmString    editReportString, saveReportString, printReportString;
    XmString    newString, saveTeXString;

    fileString = XmStringCreateSimple("File");
    
    
    menubar = XmVaCreateSimpleMenuBar(mainWindow, "menubar",
                                      XmVaCASCADEBUTTON, fileString, 'F',
                                      NULL);

    XmStringFree(fileString);

    newString = XmStringCreateSimple("New timer log");
    openString = XmStringCreateSimple("Open timer log");
    saveString = XmStringCreateSimple("Save timer log");
    saveAsString = XmStringCreateSimple("Save timer log as...");
    quitString = XmStringCreateSimple("Quit");

    editReportString = XmStringCreateSimple("Edit report");
    saveReportString = XmStringCreateSimple("Save report as text");
    saveTeXString = XmStringCreateSimple("Save report as LaTeX");
    printReportString = XmStringCreateSimple("Print report");

    XmVaCreateSimplePulldownMenu(
        menubar, "fileMenu", 0, (XtCallbackProc)FileMenuCallback,
        XmVaPUSHBUTTON, newString, 0, NULL, NULL,
        XmVaPUSHBUTTON, openString, 'O', NULL, NULL,
        XmVaPUSHBUTTON, saveString, 'S', NULL, NULL,
        XmVaPUSHBUTTON, saveAsString, 0, NULL, NULL,
        XmVaSEPARATOR,
        XmVaPUSHBUTTON, editReportString, 'E', NULL, NULL,
        XmVaPUSHBUTTON, saveReportString, 0, NULL, NULL,
        XmVaPUSHBUTTON, saveTeXString, 0, NULL, NULL,
        XmVaPUSHBUTTON, printReportString, 'P', NULL, NULL,
        XmVaSEPARATOR,
        XmVaPUSHBUTTON, quitString, 'Q', NULL, NULL,
        NULL);

    
    XmStringFree(openString);
    XmStringFree(saveString);
    XmStringFree(saveAsString);
    XmStringFree(quitString);
    XmStringFree(editReportString);
    XmStringFree(saveReportString);
    XmStringFree(saveTeXString);
    XmStringFree(printReportString);
    XmStringFree(newString);
        
    XtManageChild(menubar);
}


void main(int argc, char **argv)
{
    Widget          topLevel, mainArea, commandArea, timerArea;
    Widget          timerWindow, mainWindow, footerArea, footerSash;
    Widget          tmp;
    XmString        tmpString;
    Pixel           fg, bg;
    
    topLevel = XtVaAppInitialize(&gAppContext, "PSPTimer",
                                 NULL, 0,
                                 &argc, argv,
                                 gFallbackResources,
                                 NULL);

    mainWindow = XtVaCreateManagedWidget("mainWindow",
                                         xmMainWindowWidgetClass,
                                         topLevel,
                                         NULL);

    BuildMenus(mainWindow);

    mainArea = XtVaCreateManagedWidget("topLevelForm",
                                       xmFormWidgetClass,
                                       mainWindow,
                                       NULL);

    footerArea = XtVaCreateManagedWidget("footerForm",
                                         xmFormWidgetClass,
                                         mainArea,
                                         XmNbottomAttachment, XmATTACH_FORM,
                                         XmNbottomOffset, 0,
                                         XmNrightAttachment, XmATTACH_FORM,
                                         XmNleftAttachment, XmATTACH_FORM,
                                         NULL);

    gRightFooter = XtVaCreateManagedWidget("rightFooter",
                                           xmLabelWidgetClass,
                                           footerArea,
                                           XmNtopAttachment, XmATTACH_FORM,
                                           XmNrightAttachment, XmATTACH_FORM,
                                           XmNbottomAttachment, XmATTACH_FORM,
                                           XmNalignment, XmALIGNMENT_END,
                                           NULL);

    gLeftFooter = XtVaCreateManagedWidget("leftFooter",
                                           xmLabelWidgetClass,
                                           footerArea,
                                           XmNtopAttachment, XmATTACH_FORM,
                                           XmNleftAttachment, XmATTACH_FORM,
                                           XmNbottomAttachment, XmATTACH_FORM,
                                           XmNalignment,
                                           XmALIGNMENT_BEGINNING,
                                           NULL);    
    SetRightFooter("(None)");
    SetLeftFooter("Timer log:");


    footerSash = XtVaCreateManagedWidget("footerSash",
                                         xmSeparatorWidgetClass,
                                         mainArea,
                                         XmNbottomAttachment, XmATTACH_WIDGET,
                                         XmNbottomWidget, footerArea,
                                         XmNrightAttachment, XmATTACH_FORM,
                                         XmNleftAttachment, XmATTACH_FORM,
                                         NULL);
                                         


    commandArea = XtVaCreateManagedWidget("commandForm",
                                          xmFormWidgetClass,
                                          mainArea,
                                          XmNbottomAttachment, XmATTACH_WIDGET,
                                          XmNbottomWidget, footerSash,
                                          XmNrightAttachment, XmATTACH_FORM,
                                          XmNleftAttachment, XmATTACH_FORM,
                                          NULL);
    
    timerWindow =
        XtVaCreateManagedWidget("timerWindow",
                                xmScrolledWindowWidgetClass,
                                mainArea,
                                XmNscrollBarDisplayPolicy, XmAS_NEEDED,
                                XmNscrollingPolicy, XmAUTOMATIC,
                                XmNbottomAttachment, XmATTACH_WIDGET,
                                XmNbottomWidget, commandArea,
                                XmNrightAttachment, XmATTACH_FORM,
                                XmNleftAttachment, XmATTACH_FORM,
                                XmNtopAttachment, XmATTACH_FORM,
                                XmNscrolledWindowMarginHeight, 0,
                                XmNscrolledWindowMarginWidth, 0,
                                XmNspacing, 1,
                                NULL);
    timerArea = 
        XtVaCreateManagedWidget("timerArea", 
                                xmRowColumnWidgetClass,
                                timerWindow,
                                XmNisAligned, False,
                                XmNorientation, XmHORIZONTAL,
                                XmNnumColumns, NUM_TIMERS + 1,
                                XmNpacking, XmPACK_COLUMN,
                                XmNmarginHeight, 5,
                                XmNmarginWidth, 5,
                                NULL);

    tmpString = XmStringCreateSimple("Pause all");
    gPauseButton = XtVaCreateManagedWidget("pauseButton",
                                          xmPushButtonWidgetClass,
                                          commandArea,
                                          XmNlabelString, tmpString,
                                          XmNtopAttachment, XmATTACH_FORM,
                                          XmNbottomAttachment, XmATTACH_FORM,
                                          XmNleftAttachment, XmATTACH_FORM,
                                          XmNrightAttachment, XmATTACH_FORM,
                                          XmNbottomOffset, 6,
                                          XmNtopOffset, 2,
                                          XmNrightOffset, 2,
                                          XmNleftOffset, 2,
                                          NULL);
    XmStringFree(tmpString);

    XtVaGetValues(timerArea,
                  XtNforeground, &fg,
                  XtNbackground, &bg,
                  NULL);

    ReadPixmaps(XtScreen(timerArea),
                fg,
                bg);
    
    gTimerList[0] = CreateTimer(timerArea, NULL,
                                "planTimer", "Planning: ", 1,
                                kPlanning);
    gTimerList[1] = CreateTimer(timerArea, gTimerList[0],
                               "designTimer", "Design: ", 1,
                                kDesign); 
    gTimerList[2] = CreateTimer(timerArea, gTimerList[1],
                               "designTimer", "Design review: ", 1,
                                kDesign); 
    gTimerList[3] = CreateTimer(timerArea, gTimerList[2],
                               "codeTimer", "Coding: ", 1,
                                kCoding); 
    gTimerList[4] = CreateTimer(timerArea, gTimerList[3],
                               "codeTimer", "Code review: ", 1,
                                kCoding); 
    gTimerList[5] = CreateTimer(timerArea, gTimerList[4],
                               "compileTimer", "Compiling: ", 1,
                                kCompiling); 
    gTimerList[6] = CreateTimer(timerArea, gTimerList[5],
                               "testTimer", "Testing: ", 1,
                                kTesting); 
    gTimerList[7] = CreateTimer(timerArea, gTimerList[6],
                               "postMortemTimer", "Post Mortem: ", 1,
                                kPostmortem);
    gTimerList[8] = CreateTimer(timerArea, gTimerList[7],
                               "miscTimer1", "Other 1: ", 0,
                                kNone);
    gTimerList[9] = CreateTimer(timerArea, gTimerList[8],
                               "miscTimer2", "Other 2: ", 0,
                                kNone);

    tmp =
        XtVaCreateManagedWidget("totalTimeLabel",
                                xmLabelWidgetClass,
                                timerArea,
                                XmNalignment, XmALIGNMENT_END,
                                NULL);
    SetLabel(tmp, "Total: ");

    tmp =
        XtVaCreateManagedWidget("displayForm",
                                xmFormWidgetClass,
                                timerArea,
                                NULL);
                                
    gTotalTimeWidget =
        XtVaCreateManagedWidget("totalTime",
                                xmLabelWidgetClass,
                                tmp,
                                XmNorientation, XmHORIZONTAL,
                                XmNalignment, XmALIGNMENT_BEGINNING,
                                XmNleftAttachment, XmATTACH_FORM,
                                XmNtopAttachment, XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                NULL);
    SetLabel(gTotalTimeWidget, "0000:00");
                                
    SetDialogParent(mainWindow);
    SetDialogAppContext(gAppContext);

    CreateEventLogWindow(mainWindow);

    XtAddCallback(gPauseButton,
                  XmNactivateCallback,
                  GlobalPauseCallback,
                  NULL);

    XtRealizeWidget(topLevel);
    XtAppMainLoop(gAppContext);
}
